@@ -13,10 +13,16 @@
     if (typeof futureAccountData.then !== 'function') {
       angular.extend(this, futureAccountData);
       _.forEach(this.identities, function(identity) {
-        if (identity.fullName)
+        if (identity.fullName && identity.email)
           identity.full = identity.fullName + ' <' + identity.email + '>';
-        else
+        else if (identity.email)
           identity.full = '<' + identity.email + '>';
+        else
+          identity.full = '';
+        if (identity.signature) {
+          var element = angular.element('<div>' + identity.signature + '</div>');
+          identity.textSignature = _.map(element.contents(), 'textContent').join(' ').trim();
+        }
       });
       Account.$log.debug('Account: ' + JSON.stringify(futureAccountData, undefined, 2));
     }
@@ -96,6 +102,61 @@
     return collection;
   };
 
+  /**
+   * @memberof Account
+   * @desc Refresh the unseen count for all required mailboxes.
+   *       Starts a timer if user choose to automatically refresh folders.
+   * @param {array} [string] - the paths of the folders
+   */
+  Account.refreshUnseenCount = function(folders) {
+    var unseenCountFolders,
+        fetchAllUnseenCountFolders = (Account.$Preferences.defaults.SOGoMailFetchAllUnseenCountFolders === 1),
+        refreshViewCheck = Account.$Preferences.defaults.SOGoRefreshViewCheck;
+
+    if (fetchAllUnseenCountFolders)
+      unseenCountFolders = [];
+    else if (folders)
+      unseenCountFolders = folders;
+    else
+      throw Error('SOGoMailFetchAllUnseenCountFolders is disabled and no folders list provided');
+
+    _.forEach(Account.$accounts, function(account) {
+      if (fetchAllUnseenCountFolders) {
+        // Include all mailboxes
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          unseenCountFolders.push(mailbox.id);
+        });
+      }
+      else {
+        // Always include the INBOX
+        if (!_.includes(unseenCountFolders, account.id + '/folderINBOX'))
+          unseenCountFolders.push(account.id + '/folderINBOX');
+
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          if (angular.isDefined(mailbox.unseenCount) &&
+              !_.includes(unseenCountFolders, mailbox.id))
+            unseenCountFolders.push(mailbox.id);
+        });
+      }
+    });
+
+    Account.$$resource.post('', 'unseenCount', {mailboxes: unseenCountFolders}).then(function(data) {
+      _.forEach(Account.$accounts, function(account) {
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          if (data[mailbox.id]) {
+            mailbox.unseenCount = data[mailbox.id];
+          }
+        });
+      });
+    });
+
+    if (refreshViewCheck && refreshViewCheck != 'manually') {
+      if (Account.$refreshUnseenCount)
+        Account.$timeout.cancel(Account.$refreshUnseenCount);
+      Account.$refreshUnseenCount = Account.$timeout(angular.bind(this, Account.refreshUnseenCount, folders), refreshViewCheck.timeInterval()*1000);
+    }
+  };
+
   /**
    * @function getLength
    * @memberof Account.prototype
@@ -133,26 +194,41 @@
    * @returns a promise of the HTTP operation
    */
   Account.prototype.$getMailboxes = function(options) {
-    var _this = this;
+    var _this = this, reload = (options && options.reload);
 
-    if (this.$mailboxes && !(options && options.reload)) {
+    if (this.$mailboxes && !reload) {
       return Account.$q.when(this.$mailboxes);
     }
-    else if (this.$futureMailboxesData) {
+    else if (!reload && this.$futureMailboxesData) {
       return this.$futureMailboxesData;
     }
     else {
       this.$futureMailboxesData = Account.$Mailbox.$find(this, options).then(function(data) {
+        var previousMailboxes = _this.$flattenMailboxes({ all: true });
         _this.$mailboxes = data;
         _this.$expanded = false;
 
+        // Restore unseen count
+        var _visitForUnseencount = function(mailboxes) {
+          _.forEach(mailboxes, function(o) {
+            var previousMailbox = _.find(previousMailboxes, ['id', o.id]);
+            if (previousMailbox) {
+              o.unseenCount = previousMailbox.unseenCount;
+            }
+            if (o.children && o.children.length > 0) {
+              _visitForUnseencount(o.children);
+            }
+          });
+        };
+        _visitForUnseencount(_this.$mailboxes);
+
         // Set expanded folders from user's settings
         var expandedFolders,
-            _visit = function(mailboxes) {
+            _visitForExpanded = function(mailboxes) {
               _.forEach(mailboxes, function(o) {
                 o.$expanded = (expandedFolders.indexOf('/' + o.id) >= 0);
                 if (o.children && o.children.length > 0) {
-                  _visit(o.children);
+                  _visitForExpanded(o.children);
                 }
               });
             };
@@ -173,11 +249,12 @@
           }
           _this.$expanded = (expandedFolders.indexOf('/' + _this.id) >= 0);
           if (expandedFolders.length > 0) {
-            _visit(_this.$mailboxes);
+            _visitForExpanded(_this.$mailboxes);
           }
         }
         if (Account.$accounts)
           _this.$expanded |= (Account.$accounts.length == 1); // Always expand single account
+
         _this.$flattenMailboxes({reload: true});
 
         return _this.$mailboxes;
@@ -299,6 +376,32 @@
     });
   };
 
+  /**
+   * @function getTextSignature
+   * @memberof Account.prototype
+   * @desc Create a plain text representation of the signature for the specified identity index.
+   * @returns a plain text version of the signature
+   */
+  Account.prototype.getTextSignature = function(identity) {
+    if (identity.signature) {
+      var element = angular.element('<div>' + identity.signature + '</div>');
+      identity.textSignature = _.map(element.contents(), 'textContent').join(' ').trim();
+    } else {
+      identity.textSignature = '';
+    }
+    return identity.textSignature;
+  };
+
+  /**
+   * @function $hasCertificate
+   * @memberof Account.prototype
+   * @desc Return true if the user has a S/MIME certificate for this account
+   * @returns a boolean value
+   */
+  Account.prototype.$hasCertificate = function() {
+    return this.security && this.security.hasCertificate;
+  };
+
   /**
    * @function $certificate
    * @memberof Account.prototype
@@ -308,7 +411,7 @@
   Account.prototype.$certificate = function() {
     var _this = this;
 
-    if (this.security && this.security.hasCertificate) {
+    if (this.$hasCertificate()) {
       if (this.$$certificate)
         return Account.$q.when(this.$$certificate);
       else {
@@ -346,9 +449,16 @@
   Account.prototype.updateQuota = function(data) {
     var percent, format, description;
 
-    percent = (Math.round(data.usedSpace * 10000 / data.maxQuota) / 100);
-    format = l("quotasFormat");
-    description = format.formatted(percent, Math.round(data.maxQuota/10.24)/100);
+    if (data.maxQuota) {
+      percent = (Math.round(data.usedSpace * 10000 / data.maxQuota) / 100);
+      format = l("quotasFormat");
+      description = format.formatted(percent, Math.round(data.maxQuota/10.24)/100);
+    }
+    else if (data.maxMessages) {
+      percent = (Math.round(data.messagesCount * 10000 / data.maxMessages) / 100);
+      format = l("messageQuotasFormat");
+      description = format.formatted(percent, data.maxMessages);
+    }
 
     this.$quota = { percent: percent, description: description };
   };
@@ -434,5 +544,34 @@
       }
     });
   };
- 
+
+  /**
+   * @function $omit
+   * @memberof Account.prototype
+   * @desc Return a sanitized object used to send to the server.
+   * @return an object literal copy of the Account instance
+   */
+  Account.prototype.$omit = function () {
+    var account = {}, identities = [], defaultIdentity = false;
+
+    angular.forEach(this, function(value, key) {
+      if (key != 'constructor' && key !='identities' && key[0] != '$') {
+        account[key] = angular.copy(value);
+      }
+    });
+
+    _.forEach(this.identities, function (identity) {
+      if (!identity.isReadOnly)
+        identities.push(_.pick(identity, ['email', 'fullName', 'replyTo', 'signature', 'isDefault']));
+      if (identity.isDefault)
+        defaultIdentity = identity;
+    });
+    account.identities = identities;
+
+    if (!defaultIdentity || !account.forceDefaultIdentity)
+      delete account.forceDefaultIdentity;
+
+    return account;
+  };
+
 })();
