@@ -104,9 +104,12 @@
     var collection = [],
         // Local recursive function
         createMailboxes = function(level, mailbox) {
+          mailbox.isSentFolder = mailbox.isSentFolder || mailbox.type == 'sent';
           for (var i = 0; i < mailbox.children.length; i++) {
             mailbox.children[i].level = level;
             mailbox.children[i] = new Mailbox(account, mailbox.children[i]);
+            if (mailbox.isSentFolder)
+              mailbox.children[i].isSentFolder = true;
             createMailboxes(level+1, mailbox.children[i]);
           }
         };
@@ -162,6 +165,8 @@
       this.$isLoading = true;
       this.$messages = [];
       this.uidsMap = {};
+      this.$visibleMessages = this.$messages;
+      this.$selectedMessages = [];
     }
     angular.extend(this, data);
     if (this.path) {
@@ -230,16 +235,7 @@
    * @returns the number of messages in the mailbox
    */
   Mailbox.prototype.getLength = function() {
-    var _this = this, collapsedThread = false;
-    var visibleMessages = _.filter(this.$messages, function(msg, i) {
-      if (msg.first) {
-        collapsedThread = msg.collapsed;
-      } else if (msg.level < 0) {
-        collapsedThread = false;
-      }
-      return msg.first || collapsedThread === false;
-    });
-    return visibleMessages.length;
+    return this.$visibleMessages.length;
   };
 
   /**
@@ -249,22 +245,13 @@
    * @returns the message at the specified index
    */
   Mailbox.prototype.getItemAtIndex = function(index) {
-    var _this = this, collapsedThread = false, message;
-    var visibleMessages = _.filter(this.$messages, function(msg, i) {
-      if (msg.first) {
-        collapsedThread = msg.collapsed;
-      } else if (msg.level < 0) {
-        collapsedThread = false; // leaving the thread
-      }
-      return msg.first || collapsedThread === false;
-    });
+    var message;
 
-    if (index >= 0 && index < visibleMessages.length) {
-      message = visibleMessages[index];
+    if (index >= 0 && index < this.$visibleMessages.length) {
+      message = this.$visibleMessages[index];
       this.$lastVisibleIndex = Math.max(0, index - 3); // Magic number is NUM_EXTRA from virtual-repeater.js
-
-      if (this.$loadMessage(message.uid))
-        return message;
+      this.$loadMessage(message.uid);
+      return message; // skeleton is displayed while headers are being fetched
     }
     return null;
   };
@@ -280,23 +267,37 @@
   };
 
   /**
-   * @function $selectedMessages
+   * @function selectedMessages
    * @memberof Mailbox.prototype
    * @desc Return the messages selected by the user.
    * @returns Message instances
    */
-  Mailbox.prototype.$selectedMessages = function() {
-    return _.filter(this.$messages, function(message) { return message.selected; });
+  Mailbox.prototype.selectedMessages = function(options) {
+    if (options && options.updateCache)
+      this.$selectedMessages = _.filter(this.$messages, function(message) { return message.selected; });
+    return this.$selectedMessages;
   };
 
   /**
-   * @function $selectedCount
+   * @function selectedCount
    * @memberof Mailbox.prototype
    * @desc Return the number of messages selected by the user.
    * @returns the number of selected messages
    */
-  Mailbox.prototype.$selectedCount = function() {
-    return this.$selectedMessages().length;
+  Mailbox.prototype.selectedCount = function() {
+    return this.$selectedMessages.length;
+  };
+
+  /**
+   * @function $unselectMessages
+   * @memberof Mailbox.prototype
+   * @desc Unselect all messages.
+   */
+  Mailbox.prototype.$unselectMessages = function() {
+    _.forEach(this.$selectedMessages, function(message) {
+      message.selected = false;
+    });
+    this.$selectedMessages = [];
   };
 
   /**
@@ -307,7 +308,7 @@
    * @returns true if the specified message is displayed
    */
   Mailbox.prototype.isSelectedMessage = function(messageId) {
-    return this.selectedMessage == messageId;
+    return this.$selectedMessage == messageId;
   };
 
   /**
@@ -316,10 +317,9 @@
    * @desc Return the currently visible message.
    * @returns a Message instance or undefined if no message is displayed
    */
-  Mailbox.prototype.$selectedMessage = function() {
+  Mailbox.prototype.selectedMessage = function() {
     var _this = this;
-
-    return _.find(this.$messages, function(message) { return message.uid == _this.selectedMessage; });
+    return _.find(this.$messages, function(message) { return message.uid == _this.$selectedMessage; });
   };
 
   /**
@@ -329,7 +329,7 @@
    * @returns a number or undefined if no message is selected
    */
   Mailbox.prototype.$selectedMessageIndex = function() {
-    return this.uidsMap[this.selectedMessage];
+    return this.uidsMap[this.$selectedMessage];
   };
 
   /**
@@ -339,7 +339,7 @@
    * @returns true if the a message is selected
    */
   Mailbox.prototype.hasSelectedMessage = function() {
-    return angular.isDefined(this.selectedMessage);
+    return angular.isDefined(this.$selectedMessage);
   };
 
   /**
@@ -357,14 +357,12 @@
    * @returns a promise of the HTTP operation
    */
   Mailbox.prototype.$filter = function(sortingAttributes, filters) {
-    var _this = this, options = {};
+    var _this = this, action = 'view', options = {};
 
     if (!angular.isDefined(this.unseenCount))
       this.unseenCount = 0;
 
-    Mailbox.$timeout(function() {
-      _this.$isLoading = true;
-    });
+    this.$isLoading = true;
 
     if (Mailbox.$refreshTimeout)
       Mailbox.$timeout.cancel(Mailbox.$refreshTimeout);
@@ -375,7 +373,7 @@
 
     angular.extend(options, { sortingAttributes: Mailbox.$query });
     if (angular.isDefined(filters)) {
-      options.filters = _.reject(filters, function(filter) {
+      options.filters = _.reject(angular.copy(filters), function(filter) {
         return !filter.searchInput || filter.searchInput.length === 0;
       });
       // Decompose filters that match two fields
@@ -391,6 +389,10 @@
         }
       });
     }
+    else if (!sortingAttributes && this.$syncToken) {
+      action = 'changes';
+      options.syncToken = this.$syncToken;
+    }
 
     // Restart the refresh timer, if needed
     if (!Mailbox.$virtualMode) {
@@ -401,14 +403,14 @@
       }
     }
 
-    var futureMailboxData = Mailbox.$$resource.post(this.id, 'view', options);
+    var futureMailboxData = Mailbox.$$resource.post(this.id, action, options);
     return this.$unwrap(futureMailboxData);
   };
 
   /**
    * @function $loadMessage
    * @memberof Mailbox.prototype
-   * @desc Check if the message is loaded and in any case, fetch more messages headers from the server.
+   * @desc Check if the message headers are loaded and in any case, fetch more messages headers from the server.
    * @returns true if the message metadata are already fetched
    */
   Mailbox.prototype.$loadMessage = function(messageId) {
@@ -455,6 +457,7 @@
           else {
             // Message at this index will be loaded
             uids.push(this.$messages[startIndex].uid);
+            //console.debug('loading ' + this.$messages[startIndex].uid);
             this.$messages[startIndex].loading = true;
           }
         }
@@ -735,15 +738,10 @@
    * @memberof Mailbox.prototype
    * @desc Delete multiple messages from Mailbox object.
    * @param {string[]} uids - the messages uids
-   * @param {object[]} messages - the Message instances
    * @return the index of the first deleted message
    */
-  Mailbox.prototype.$_deleteMessages = function(uids, messages) {
-    var _this = this, selectedUIDs, _$messages, unseen, firstIndex = this.$messages.length;
-
-    // Decrement the unseen count
-    unseen = _.filter(messages, function(message, i) { return !message.isread; });
-    this.unseenCount -= unseen.length;
+  Mailbox.prototype.$_deleteMessages = function(uids) {
+    var _this = this, firstIndex = this.$messages.length;
 
     // Remove messages from $messages and uidsMap
     _.forEachRight(this.$messages, function(message, index) {
@@ -753,8 +751,8 @@
       if (selectedIndex > -1) {
         uids.splice(selectedIndex, 1);
         delete _this.uidsMap[message.uid];
-        if (message.uid == _this.selectedMessage)
-          delete _this.selectedMessage;
+        if (message.uid == _this.$selectedMessage)
+          delete _this.$selectedMessage;
         _this.$messages.splice(index, 1);
         if (index < firstIndex)
           firstIndex = index;
@@ -764,6 +762,10 @@
       }
     });
 
+    if (this.threaded) {
+      this.updateVisibleMessages();
+    }
+
     // Return the index of the first deleted message
     return firstIndex;
   };
@@ -784,24 +786,29 @@
     // Recursive function to synchronously delete batch of messages
     function _deleteMessages(start, end) {
       var currentUids = uids.slice(start, end),
-          currentMessages = messages.slice(start, end),
           data = { uids: currentUids };
       if (options) angular.extend(data, options);
       return Mailbox.$$resource.post(_this.id, 'batchDelete', data).then(function(data) {
+        if (data.unseenCount) {
+          _this.unseenCount = data.unseenCount;
+        }
         if (end < uids.length) {
-          _this.$_deleteMessages(currentUids, currentMessages);
+          _this.$_deleteMessages(currentUids);
           return _deleteMessages(end, Math.min(end + batchSize, uids.length));
         }
         else {
           // Last API call; update inbox quota
           if (data.quotas)
             _this.$account.updateQuota(data.quotas);
-          return _this.$_deleteMessages(currentUids, currentMessages);
+          return _this.$_deleteMessages(currentUids);
         }
       });
     }
 
-    return _deleteMessages(0, Math.min(batchSize, uids.length));
+    return _deleteMessages(0, Math.min(batchSize, uids.length)).then(function(firstIndex) {
+      _this.$selectedMessages = []; // reset selection
+      return firstIndex;
+    });
   };
 
   /**
@@ -847,8 +854,12 @@
 
     uids = _.map(messages, 'uid');
     return Mailbox.$$resource.post(this.id, 'moveMessages', {uids: uids, folder: folder})
-      .then(function() {
-        return _this.$_deleteMessages(uids, messages);
+      .then(function(data) {
+        if (data.unseenCount) {
+          _this.unseenCount = data.unseenCount;
+        }
+        _this.$selectedMessages = []; // reset selection
+        return _this.$_deleteMessages(uids);
       });
   };
 
@@ -934,6 +945,26 @@
     return mailbox;
   };
 
+  /**
+   * @function updateVisibleMessages
+   * @memberof Mailbox.prototype
+   * @desc Update list of visible messages when in threaded mode.
+   */
+  Mailbox.prototype.updateVisibleMessages = function() {
+    var collapsedThread = false;
+
+    if (this.threaded) {
+      this.$visibleMessages = _.filter(this.$messages, function(msg, i) {
+        if (msg.first) {
+          collapsedThread = msg.collapsed;
+        } else if (msg.level < 0) {
+          collapsedThread = false; // leaving the thread
+        }
+        return msg.first || collapsedThread === false;
+      });
+    }
+  };
+
   /**
    * @function $unwrap
    * @memberof Mailbox.prototype
@@ -947,21 +978,54 @@
 
     this.$futureMailboxData = futureMailboxData;
     this.$futureMailboxData.then(function(data) {
-      var selectedMessages = _.map(_this.$selectedMessages(), 'uid');
+      var selectedMessages = _.map(_this.$selectedMessages, 'uid');
       Mailbox.$timeout(function() {
-        var uids, headers;
+        var uids, headers, headersFields, msgObject, hasNewMessages = false;
 
         if (!data.uids || _this.$topIndex > data.uids.length - 1)
           _this.$topIndex = 0;
+        if (data.syncToken)
+          _this.$syncToken = data.syncToken;
 
-        _this.init(data);
+        if (data.deleted) {
+          _.forEachRight(data.deleted, function(uid, i) {
+            var j = _this.uidsMap[uid.toString()];
+            if (j < 0 || !_this.$messages[j])
+              // Unkown message
+              data.deleted.splice(i, 1);
+          });
+          if (data.deleted.length)
+            _this.$_deleteMessages(data.deleted);
+        }
+        if (data.changed) {
+          var i = 0, j;
+          _.forEach(data.changed, function(uid) {
+            if (angular.isUndefined(_this.uidsMap[uid.toString()])) {
+              // New messsage; update map of UID <=> index
+              _this.uidsMap[uid] = i;
+              _this.$messages.splice(i, 0, {uid: uid});
+              hasNewMessages = true;
+              i++;
+            }
+          });
 
-        if (_this.uids) {
-          Mailbox.$log.debug('unwrapping ' + _this.uids.length + ' messages');
+          if (i > 0) {
+            // New messages received, update uidsMap for existing messages
+            for (j = i; j < _this.$messages.length; j++) {
+              msgObject = _this.$messages[j];
+              _this.uidsMap[msgObject.uid] += i;
+            }
+          }
+        }
+        if (data.unseenCount) {
+          _this.unseenCount = data.unseenCount;
+        }
 
-          // First entry of 'headers' are keys
-          headers = _.invokeMap(_this.headers[0], 'toLowerCase');
-          _this.headers.splice(0, 1);
+        if (data.uids) {
+          // Initialization phase, we received complete list of UIDs
+          Mailbox.$log.debug('unwrapping ' + data.uids.length + ' messages');
+
+          _this.init(data);
 
           // First entry of 'uids' are keys when threaded view is enabled
           if (_this.threaded) {
@@ -969,9 +1033,9 @@
             _this.uids.splice(0, 1);
           }
 
-          // Instanciate Message objects
+          // Populate $messages with object literals
           _.reduce(_this.uids, function(msgs, msg, i) {
-            var data, msgObject;
+            var data;
             if (_this.threaded) {
               data = _.zipObject(uids, msg);
               if (data.first === 1) {
@@ -987,35 +1051,53 @@
                   data.collapsed = true;
                 }
               }
+              else if (!isNaN(data.level) && data.level >= 0) {
+                data.threadMember = true;
+              }
+
             } else {
-              data = {uid: msg.toString()};
+              data = {uid: msg};
             }
 
             // Build map of UID <=> index
             _this.uidsMap[data.uid] = i;
 
-            msgObject = new Mailbox.$Message(_this.$account.id, _this, data, true);
-
             // Restore selection
-            msgObject.selected = selectedMessages.indexOf(msgObject.uid) > -1;
+            data.selected = selectedMessages.indexOf(data.uid) > -1;
 
-            msgs.push(msgObject);
+            // Add an object literal to be used later to create a Message object
+            msgs.push(data);
 
             return msgs;
           }, _this.$messages);
+        }
 
-          // Extend Message objects with received headers
-          _.forEach(_this.headers, function(data) {
-            var msg = _.zipObject(headers, data),
+        if (data.headers) {
+          // First entry of 'headers' are keys
+          headersFields = _.invokeMap(data.headers.splice(0, 1)[0], 'toLowerCase');
+          headers = data.headers;
+
+          // Instanciate or extend Message objects with received headers
+          _.forEach(headers, function(data) {
+            var msg = _.zipObject(headersFields, data),
                 i = _this.uidsMap[msg.uid.toString()];
+            if (!(_this.$messages[i] instanceof Mailbox.$Message)) {
+              _this.$messages[i] = new Mailbox.$Message(_this.$account.id, _this, _this.$messages[i], true);
+            }
             _this.$messages[i].init(msg);
           });
         }
+
+        if (hasNewMessages && _this.threaded) {
+          _this.updateVisibleMessages();
+        }
+
         Mailbox.$log.debug('mailbox ' + _this.id + ' ready');
         _this.$isLoading = false;
         deferred.resolve(_this.$messages);
       });
     }, function(data) {
+      Mailbox.$log.error(data);
       angular.extend(_this, data);
       _this.isError = true;
       _this.$isLoading = false;
@@ -1045,9 +1127,15 @@
             messageHeaders = _.zipObject(headers, messageHeaders);
             j = _this.uidsMap[messageHeaders.uid.toString()];
             if (angular.isDefined(j)) {
+              if (!(_this.$messages[j] instanceof Mailbox.$Message)) {
+                _this.$messages[j] = new Mailbox.$Message(_this.$account.id, _this, _this.$messages[j], true);
+              }
               _this.$messages[j].init(messageHeaders);
             }
           });
+          if (_this.threaded) {
+            _this.updateVisibleMessages();
+          }
         }
       });
     });
