@@ -46,10 +46,11 @@
 
       // Update window's title with unseen messages count of selected mailbox
       $scope.$watch(function() { return vm.selectedFolder.unseenCount; }, function(unseenCount) {
-        var title = defaultWindowTitle + ' - ';
+        var title = '';
         if (unseenCount)
           title += '(' + unseenCount + ') ';
         title += vm.selectedFolder.$displayName;
+        title += ' | ' + defaultWindowTitle;
         $window.document.title = title;
       });
     };
@@ -123,6 +124,8 @@
     }
 
     function _compactBeforeUnload(event) {
+      if (Mailbox.$virtualMode)
+        return true;
       return vm.selectedFolder.$compact();
     }
 
@@ -148,6 +151,11 @@
       return Mailbox.$query.asc;
     };
 
+    this.refresh = function () {
+      Preferences.pollInbox();
+      this.selectedFolder.$filter();
+    };
+
     this.searchMode = function($event) {
       vm.mode.search = true;
       focus('search');
@@ -157,10 +165,10 @@
 
     this.cancelSearch = function() {
       vm.mode.search = false;
-      vm.selectedFolder.$filter().then(function() {
-        if (vm.selectedFolder.selectedMessage) {
+      vm.selectedFolder.$filter(vm.service.$query).then(function() {
+        if (vm.selectedFolder.$selectedMessage) {
           $timeout(function() {
-            vm.selectedFolder.$topIndex = vm.selectedFolder.uidsMap[vm.selectedFolder.selectedMessage];
+            vm.selectedFolder.$topIndex = vm.selectedFolder.uidsMap[vm.selectedFolder.$selectedMessage];
           });
         }
       });
@@ -184,8 +192,6 @@
               targetEvent: $event,
               clickOutsideToClose: false,
               escapeToClose: false,
-              scope: $scope,
-              preserveScope: true,
               templateUrl: 'UIxMailEditor',
               controller: 'MessageEditorController',
               controllerAs: 'editor',
@@ -193,6 +199,7 @@
                 return onCompleteDeferred.resolve(element);
               },
               locals: {
+                stateParent: $scope,
                 stateAccount: vm.account,
                 stateMessage: message,
                 onCompletePromise: function () {
@@ -313,14 +320,14 @@
           selectedIndex, nextSelectedIndex, i;
 
       if (!message)
-        message = folder.$selectedMessage();
+        message = folder.selectedMessage();
       if (!message)
         return true;
+
       message.selected = !message.selected;
-      vm.mode.multiple += message.selected? 1 : -1;
 
       // Select closest range of messages when shift key is pressed
-      if ($event.shiftKey && folder.$selectedCount() > 1) {
+      if ($event.shiftKey && folder.selectedCount() > 0) {
         selectedIndex = folder.uidsMap[message.uid];
         // Search for next selected message above
         nextSelectedIndex = selectedIndex - 2;
@@ -342,6 +349,8 @@
         }
       }
 
+      folder.selectedMessages({ updateCache: true });
+      vm.mode.multiple = vm.selectedFolder.selectedCount();
       $event.preventDefault();
       $event.stopPropagation();
     };
@@ -361,7 +370,7 @@
     // This function must not be called in virtual mode.
     function _unselectMessage(message, index) {
       var nextMessage, previousMessage, nextIndex = index;
-      vm.mode.multiple = vm.selectedFolder.$selectedCount();
+      vm.mode.multiple = vm.selectedFolder.selectedCount();
       if (message) {
         // Select either the next or previous message
         if (index > 0) {
@@ -388,21 +397,15 @@
           $state.go('mail.account.mailbox');
         }
       }
-      else {
-        $timeout(function() {
-          console.warn('go to mailbox');
-          $state.go('mail.account.mailbox');
-        });
-      }
     }
 
     this.confirmDeleteSelectedMessages = function($event) {
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
 
       if (vm.messageDialog === null && _.size(selectedMessages) > 0)
         vm.messageDialog = Dialog.confirm(l('Confirmation'),
-                                       l('Are you sure you want to delete the selected messages?'),
-                                       { ok: l('Delete') })
+                                            l('Are you sure you want to delete the selected messages?'),
+                                            { ok: l('Delete') })
         .then(function() {
           var deleteSelectedMessage = vm.selectedFolder.hasSelectedMessage();
           vm.selectedFolder.$deleteMessages(selectedMessages).then(function(index) {
@@ -421,18 +424,22 @@
                                            l('The messages could not be moved to the trash folder. Would you like to delete them immediately?'),
                                            { ok: l('Delete') })
               .then(function() {
-                vm.selectedFolder.$deleteMessages(selectedMessages, { withoutTrash: true }).then(function(index) {
-                  if (Mailbox.$virtualMode) {
-                    // When performing an advanced search, we refresh the view if the selected message
-                    // was deleted, but only once all promises have completed.
-                    if (deleteSelectedMessage)
-                      $state.go('mail.account.virtualMailbox');
-                  }
-                  else {
-                    // In normal mode, we immediately unselect the selected message.
-                    _unselectMessage(deleteSelectedMessage, index);
-                  }
-                });
+                vm.selectedFolder.$deleteMessages(selectedMessages, { withoutTrash: true })
+                  .then(function(index) {
+                    if (Mailbox.$virtualMode) {
+                      // When performing an advanced search, we refresh the view if the selected message
+                      // was deleted, but only once all promises have completed.
+                      if (deleteSelectedMessage)
+                        $state.go('mail.account.virtualMailbox');
+                    }
+                    else {
+                      // In normal mode, we immediately unselect the selected message.
+                      _unselectMessage(deleteSelectedMessage, index);
+                    }
+                  })
+                  .finally(function() {
+                    vm.messageDialog = null;
+                  });
               });
           });
         })
@@ -445,9 +452,10 @@
 
     this.markOrUnMarkMessagesAsJunk = function() {
       var moveSelectedMessage = vm.selectedFolder.hasSelectedMessage();
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
       if (_.size(selectedMessages) === 0 && moveSelectedMessage)
-        selectedMessages = [vm.selectedFolder.$selectedMessage()];
+        // No selection, user has pressed keyboard shortcut
+        selectedMessages = [vm.selectedFolder.selectedMessage()];
       if (_.size(selectedMessages) > 0)
         vm.selectedFolder.$markOrUnMarkMessagesAsJunk(selectedMessages).then(function() {
           var dstFolder = '/' + vm.account.id + '/folderINBOX';
@@ -470,12 +478,12 @@
     };
 
     this.copySelectedMessages = function(dstFolder) {
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
       if (_.size(selectedMessages) > 0)
         vm.selectedFolder.$copyMessages(selectedMessages, '/' + dstFolder).then(function() {
           $mdToast.show(
             $mdToast.simple()
-              .content(l('%{0} message(s) copied', vm.selectedFolder.$selectedCount()))
+              .textContent(l('%{0} message(s) copied', vm.selectedFolder.selectedCount()))
               .position('top right')
               .hideDelay(2000));
         });
@@ -483,13 +491,13 @@
 
     this.moveSelectedMessages = function(dstFolder) {
       var moveSelectedMessage = vm.selectedFolder.hasSelectedMessage();
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
-      var count = vm.selectedFolder.$selectedCount();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
+      var count = vm.selectedFolder.selectedCount();
       if (_.size(selectedMessages) > 0)
         vm.selectedFolder.$moveMessages(selectedMessages, '/' + dstFolder).then(function(index) {
           $mdToast.show(
             $mdToast.simple()
-              .content(l('%{0} message(s) moved', count))
+              .textContent(l('%{0} message(s) moved', count))
               .position('top right')
               .hideDelay(2000));
           if (Mailbox.$virtualMode) {
@@ -509,8 +517,11 @@
       var count = 0;
       _.forEach(_currentMailboxes(), function(folder) {
         var i = 0, length = folder.$messages.length;
-        for (; i < length; i++)
+        folder.$selectedMessages = [];
+        for (; i < length; i++) {
           folder.$messages[i].selected = true;
+          folder.$selectedMessages.push(folder.$messages[i]);
+        }
         count += length;
       });
       vm.mode.multiple = count;
@@ -518,6 +529,7 @@
 
     this.unselectMessages = function() {
       _.forEach(_currentMailboxes(), function(folder) {
+        folder.$selectedMessages = [];
         _.forEach(folder.$messages, function(message) {
           message.selected = false;
         });
@@ -526,7 +538,7 @@
     };
 
     this.markSelectedMessagesAsFlagged = function() {
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
       if (_.size(selectedMessages) > 0)
         vm.selectedFolder.$flagMessages(selectedMessages, '\\Flagged', 'add').then(function(messages) {
           _.forEach(messages, function(message) {
@@ -536,7 +548,7 @@
     };
 
     this.markSelectedMessagesAsUnread = function() {
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
       if (_.size(selectedMessages) > 0) {
         vm.selectedFolder.$flagMessages(selectedMessages, 'seen', 'remove').then(function(messages) {
           _.forEach(messages, function(message) {
@@ -549,7 +561,7 @@
     };
 
     this.markSelectedMessagesAsRead = function() {
-      var selectedMessages = vm.selectedFolder.$selectedMessages();
+      var selectedMessages = vm.selectedFolder.selectedMessages();
       if (_.size(selectedMessages) > 0) {
         vm.selectedFolder.$flagMessages(selectedMessages, 'seen', 'add').then(function(messages) {
           _.forEach(messages, function(message) {
