@@ -6,8 +6,8 @@
   /**
    * @ngInject
    */
-  MessageController.$inject = ['$window', '$scope', '$q', '$state', '$mdMedia', '$mdDialog', 'sgConstant', 'stateAccounts', 'stateAccount', 'stateMailbox', 'stateMessage', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'ImageGallery', 'sgFocus', 'Dialog', 'Preferences', 'Calendar', 'Component', 'Account', 'Mailbox', 'Message'];
-  function MessageController($window, $scope, $q, $state, $mdMedia, $mdDialog, sgConstant, stateAccounts, stateAccount, stateMailbox, stateMessage, sgHotkeys, encodeUriFilter, sgSettings, ImageGallery, focus, Dialog, Preferences, Calendar, Component, Account, Mailbox, Message) {
+  MessageController.$inject = ['$window', '$scope', '$q', '$state', '$mdMedia', '$mdDialog', '$mdPanel', 'sgConstant', 'stateAccounts', 'stateAccount', 'stateMailbox', 'stateMessage', 'sgHotkeys', 'encodeUriFilter', 'sgSettings', 'ImageGallery', 'sgFocus', 'Dialog', 'Preferences', 'Calendar', 'Component', 'Account', 'Mailbox', 'Message', 'AddressBook', 'Card'];
+  function MessageController($window, $scope, $q, $state, $mdMedia, $mdDialog, $mdPanel, sgConstant, stateAccounts, stateAccount, stateMailbox, stateMessage, sgHotkeys, encodeUriFilter, sgSettings, ImageGallery, focus, Dialog, Preferences, Calendar, Component, Account, Mailbox, Message, AddressBook, Card) {
     var vm = this, popupWindow = null, hotkeys = [];
 
     this.$onInit = function() {
@@ -27,7 +27,8 @@
       this.service = Message;
       this.tags = { searchText: '', selected: '' };
       this.showFlags = stateMessage.flags && stateMessage.flags.length > 0;
-      this.$showDetailedRecipients = false;
+      this.$alwaysShowDetailedRecipients = (!stateMessage.to || stateMessage.to.length < 5) && (!stateMessage.cc || stateMessage.cc.length < 5);
+      this.$showDetailedRecipients = this.$alwaysShowDetailedRecipients;
       this.showRawSource = false;
 
       _registerHotkeys(hotkeys);
@@ -147,7 +148,7 @@
         keys.push(sgHotkeys.createHotkey({
           key: hotkey,
           callback: _unlessInDialog(function($event) {
-            if (vm.mailbox.$selectedCount() === 0)
+            if (vm.mailbox.selectedCount() === 0)
               vm.deleteMessage();
             $event.preventDefault();
           }),
@@ -197,6 +198,108 @@
       $event.preventDefault();
     };
 
+    this.focusChip = function($event) {
+      var chipElement = $event.target;
+      while (chipElement.tagName !== 'MD-CHIP') {
+        chipElement = chipElement.parentNode;
+      }
+      chipElement.classList.add('md-focused');
+    };
+
+    this.blurChip = function($event) {
+      var chipElement = $event.target;
+      while (chipElement.tagName !== 'MD-CHIP') {
+        chipElement = chipElement.parentNode;
+      }
+      chipElement.classList.remove('md-focused');
+      if ($event.relatedTarget && $event.relatedTarget.tagName === 'MD-CHIP-TEMPLATE') {
+        // Moving to another chip; close menu
+        vm.panel.close();
+      }
+    };
+
+    this.selectRecipient = function(recipient, $event) {
+      // Fetch addressbooks list
+      AddressBook.$findAll([]);
+
+      var targetElement = $event.target;
+
+      var panelPosition = $mdPanel.newPanelPosition()
+          .relativeTo(targetElement)
+          .addPanelPosition(
+            $mdPanel.xPosition.ALIGN_START,
+            $mdPanel.yPosition.ALIGN_TOPS
+          );
+
+      var panelAnimation = $mdPanel.newPanelAnimation()
+          .openFrom(targetElement)
+          .duration(100)
+          .withAnimation($mdPanel.animation.FADE);
+
+      var config = {
+        attachTo: angular.element(document.body),
+        locals: {
+          recipient: recipient,
+          addressbooks: AddressBook.$addressbooks,
+          subscriptions: AddressBook.$subscriptions,
+          newMessage: angular.bind(this, this.newMessage)
+        },
+        bindToController: true,
+        controller: MenuController,
+        controllerAs: '$menuCtrl',
+        position: panelPosition,
+        animation: panelAnimation,
+        targetEvent: $event,
+        templateUrl: 'UIxMailViewRecipientMenu',
+        trapFocus: true,
+        clickOutsideToClose: true,
+        escapeToClose: true,
+        focusOnOpen: false
+      };
+
+      $mdPanel.open(config)
+        .then(function(panelRef) {
+          vm.panel = panelRef;
+          // Automatically close panel when clicking inside of it
+          panelRef.panelEl.one('click', function() {
+            panelRef.close();
+          });
+        });
+
+      MenuController.$inject = ['mdPanelRef', '$state', '$mdToast'];
+      function MenuController(mdPanelRef, $state, $mdToast) {
+        this.onKeyDown = function($event) {
+          if ($event.which === 9) { // Tab
+            mdPanelRef.close();
+          }
+        };
+
+        this.newCard = function(recipient, addressbookId) {
+          var card = new Card({
+            pid: addressbookId,
+            c_cn: recipient.name,
+            emails: [{ value: recipient.email }]
+          });
+          card.$id().then(function(id) {
+            card.$save().then(function() {
+              // Show success toast when action succeeds
+              $mdToast.show(
+                $mdToast.simple()
+                  .textContent(l('Successfully created card'))
+                  .position('top right')
+                  .hideDelay(2000));
+            });
+          });
+          mdPanelRef.close();
+        };
+      }
+
+      if (targetElement.tagName === 'A') {
+        $event.stopPropagation();
+        $event.preventDefault();
+      }
+    };
+
     this.filterMailtoLinks = function($event) {
       var href, match, to, cc, bcc, subject, body, data;
       if ($event.target.tagName == 'A' && 'href' in $event.target.attributes) {
@@ -211,7 +314,8 @@
 
     this.deleteMessage = function() {
       var mailbox, message, state, nextMessage, previousMessage,
-          parentCtrls = $parentControllers();
+          parentCtrls = $parentControllers(),
+          $timeout = this.service.$timeout;
 
       if (parentCtrls.messageCtrl) {
         mailbox = parentCtrls.mailboxCtrl.selectedFolder;
@@ -227,7 +331,7 @@
         mailbox = Mailbox.selectedFolder; // the VirtualMailbox instance
       }
 
-      mailbox.$deleteMessages([message]).then(function(index) {
+      function _success(index) {
         var nextIndex = index;
         // Remove message object from scope
         message = null;
@@ -257,21 +361,41 @@
                 state.go('mail.account.virtualMailbox.message', {mailboxId: encodeUriFilter(nextMessage.$mailbox.path), messageId: nextMessage.uid});
               else
                 state.go('mail.account.mailbox.message', {messageId: nextMessage.uid});
-              if (nextIndex < mailbox.$topIndex)
-                mailbox.$topIndex = nextIndex;
-              else if (nextIndex > mailbox.$lastVisibleIndex)
-                mailbox.$topIndex = nextIndex - (mailbox.$lastVisibleIndex - mailbox.$topIndex);
+              $timeout(function() {
+                if (nextIndex < mailbox.$topIndex)
+                  mailbox.$topIndex = nextIndex;
+                else if (nextIndex > mailbox.$lastVisibleIndex)
+                  mailbox.$topIndex = nextIndex - (mailbox.$lastVisibleIndex - mailbox.$topIndex);
+              });
             }
             else {
               state.go('mail.account.mailbox').then(function() {
                 message = null;
-                delete mailbox.selectedMessage;
+                delete mailbox.$selectedMessage;
               });
             }
           }
           catch (error) {}
         }
         vm.closePopup();
+      }
+
+      mailbox.$deleteMessages([message]).then(_success, function(response) {
+        _messageDialog(
+            Dialog.confirm(l('Warning'),
+                           l('The message could not be moved to the trash folder. Would you like to delete it immediately?'),
+                           { ok: l('Delete') })
+            .then(function() {
+              mailbox.$deleteMessages([message], { withoutTrash: true })
+                .then(_success)
+                .finally(function() {
+                  _messageDialog(null);
+                });
+            })
+            .finally(function() {
+              _messageDialog(null);
+            })
+        );
       });
     };
 
@@ -285,8 +409,6 @@
               targetEvent: $event,
               clickOutsideToClose: false,
               escapeToClose: false,
-              scope: $scope,
-              preserveScope: true,
               templateUrl: 'UIxMailEditor',
               controller: 'MessageEditorController',
               controllerAs: 'editor',
@@ -294,6 +416,7 @@
                 return onCompleteDeferred.resolve(element);
               },
               locals: {
+                stateParent: $scope,
                 stateAccount: vm.account,
                 stateMessage: message,
                 onCompletePromise: function () {
@@ -323,7 +446,7 @@
       var destination = Mailbox.$virtualMode ? 'mail.account.virtualMailbox' : 'mail.account.mailbox';
       $state.go(destination).then(function() {
         vm.message = null;
-        delete stateMailbox.selectedMessage;
+        delete stateMailbox.$selectedMessage;
       });
     };
 
@@ -383,8 +506,10 @@
     };
 
     this.newMessage = function($event, mailto) {
-      $event.stopPropagation();
-      $event.preventDefault();
+      if ($event.target.tagName === 'A') {
+        $event.stopPropagation();
+        $event.preventDefault();
+      }
       this.account.$newMessage({ mailto: mailto }).then(function(message) {
         _showMailEditor($event, message);
       });
