@@ -208,12 +208,12 @@
    * @see {@link Calendar.$getComponent}
    */
   Component.$find = function(calendarId, componentId, occurrenceId) {
-    var futureComponentData, path = [calendarId, encodeURIComponent(componentId)];
+    var futureComponentData, path = [calendarId, componentId];
 
     if (occurrenceId)
       path.push(occurrenceId);
 
-    futureComponentData = this.$$resource.fetch(path.join('/'), 'view');
+    futureComponentData = this.$$resource.fetch(path, 'view');
 
     return new Component(futureComponentData);
   };
@@ -441,7 +441,7 @@
           }, components);
         }
 
-        Component.$log.debug('list of ' + type + ' ready (' + components.length + ')');
+        Component.$log.debug('list of ' + type + ' ready (' + _.size(components) + ')');
 
         // Save the list of components to the object model
         Component['$' + type] = components;
@@ -483,7 +483,7 @@
 
     return new Date(parseInt(date[0]), parseInt(date[1]) - 1, parseInt(date[2]),
                     parseInt(time[0]), parseInt(time[1]), 0, 0);
-    };
+  };
 
   /**
    * @function init
@@ -694,13 +694,25 @@
   /**
    * @function isInvitationOccurrence
    * @memberof Component.prototype
-   * @desc Check if the component an invitation and an occurrence of a recurrent component
+   * @desc Check if the component is an invitation and an occurrence of a recurrent component
    * @returns true or false
    */
   Component.prototype.isInvitationOccurrence = function() {
     return (this.occurrenceId && this.userHasRSVP);
   };
 
+  /**
+   * @function isMovable
+   * @memberof Component.prototype
+   * @desc Return true if the component can be moved to a different calendar which occurs in two cases:
+   *       - the component is editable, ie is owned by the current user;
+   *       - the component is an invitation and the current user is an attendee.
+   * @returns true or false
+   */
+  Component.prototype.isMovable = function() {
+    return (!this.isReadOnly || this.userHasRSVP);
+  };
+
   /**
    * @function showPercentComplete
    * @memberof Component.prototype
@@ -759,6 +771,13 @@
    * @param {number} delta - the number of minutes
    */
   Component.prototype.setDelta = function(delta) {
+    if (delta < 0) {
+      var start = new Date(this.start.getTime());
+      start.setMinutes(Math.round(start.getMinutes()/15)*15);
+      start.addMinutes(delta);
+      this.start = start;
+      delta *= -1;
+    }
     this.delta = delta;
     this.end = new Date(this.start.getTime());
     this.end.setMinutes(Math.round(this.end.getMinutes()/15)*15);
@@ -914,7 +933,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.$reply = function() {
-    var _this = this, data, path = [this.pid, encodeURIComponent(this.id)];
+    var _this = this, data, path = [this.pid, this.id];
 
     if (this.occurrenceId)
       path.push(this.occurrenceId);
@@ -922,10 +941,11 @@
     data = {
       reply: this.reply,
       delegatedTo: this.delegatedTo,
-      alarm: this.$hasAlarm? this.alarm : {}
+      alarm: this.$hasAlarm? this.alarm : {},
+      classification: this.classification
     };
 
-    return Component.$$resource.save(path.join('/'), data, { action: 'rsvpAppointment' })
+    return Component.$$resource.save(path, data, { action: 'rsvpAppointment' })
       .then(function(data) {
         // Make a copy of the data for an eventual reset
         _this.$shadowData = _this.$omit();
@@ -940,7 +960,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.$adjust = function(params) {
-    var path = [this.pid, encodeURIComponent(this.id)];
+    var path = [this.pid, this.id];
 
     if (_.every(_.values(params), function(v) { return v === 0; }))
       // No changes
@@ -951,7 +971,7 @@
 
     Component.$log.debug('adjust ' + path.join('/') + ' ' + JSON.stringify(params));
 
-    return Component.$$resource.save(path.join('/'), params, { action: 'adjust' });
+    return Component.$$resource.save(path, params, { action: 'adjust' });
   };
 
   /**
@@ -1036,7 +1056,7 @@
     }
 
     // Build URL
-    path = [this.pid, encodeURIComponent(this.id)];
+    path = [this.pid, this.id];
 
     if (this.isNew)
       options = { action: 'saveAs' + this.type.capitalize() };
@@ -1046,7 +1066,7 @@
 
     angular.extend(component, extraAttributes);
 
-    return Component.$$resource.save(path.join('/'), component, options)
+    return Component.$$resource.save(path, component, options)
       .then(function(data) {
         // Make a copy of the data for an eventual reset
         _this.$shadowData = _this.$omit();
@@ -1061,12 +1081,12 @@
    * @param {boolean} occurrenceOnly - delete this occurrence only
    */
   Component.prototype.remove = function(occurrenceOnly) {
-    var _this = this, path = [this.pid, encodeURIComponent(this.id)];
+    var _this = this, path = [this.pid, this.id];
 
     if (occurrenceOnly && this.occurrenceId)
       path.push(this.occurrenceId);
 
-    return Component.$$resource.remove(path.join('/'));
+    return Component.$$resource.remove(path);
   };
 
   /**
@@ -1158,7 +1178,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.copyTo = function(calendar) {
-    return Component.$$resource.post(this.pid + '/' + encodeURIComponent(this.id), 'copy', {destination: calendar});
+    return Component.$$resource.post([this.pid, this.id], 'copy', {destination: calendar});
   };
 
   /**
@@ -1169,7 +1189,7 @@
    * @returns a promise of the HTTP operation
    */
   Component.prototype.moveTo = function(calendar) {
-    return Component.$$resource.post(this.pid + '/' + encodeURIComponent(this.id), 'move', {destination: calendar});
+    return Component.$$resource.post([this.pid, this.id], 'move', {destination: calendar});
   };
 
   Component.prototype.toString = function() {
