@@ -128,10 +128,11 @@
       // Retrieve master event
       component = Calendar.$get(this.component.pid).$getComponent(this.component.id);
       component.$futureComponentData.then(function() {
-        // Propagate the participant status and alarm to the master event
+        // Propagate the participant status, classification and alarm to the master event
         component.reply = vm.component.reply;
         component.delegatedTo = vm.component.delegatedTo;
         component.$hasAlarm = vm.component.$hasAlarm;
+        component.classification = vm.component.classification;
         component.alarm = vm.component.alarm;
         // Send reply to the server
         vm.reply(component);
@@ -205,8 +206,8 @@
   /**
    * @ngInject
    */
-  ComponentEditorController.$inject = ['$rootScope', '$scope', '$log', '$timeout', '$window', '$element', '$mdDialog', '$mdToast', 'sgFocus', 'User', 'CalendarSettings', 'Calendar', 'Component', 'Attendees', 'AddressBook', 'Card', 'Preferences', 'stateComponent'];
-  function ComponentEditorController($rootScope, $scope, $log, $timeout, $window, $element, $mdDialog, $mdToast, focus, User, CalendarSettings, Calendar, Component, Attendees, AddressBook, Card, Preferences, stateComponent) {
+  ComponentEditorController.$inject = ['$rootScope', '$scope', '$q', '$log', '$timeout', '$window', '$element', '$mdDialog', '$mdToast', 'sgFocus', 'User', 'CalendarSettings', 'Calendar', 'Component', 'Attendees', 'AddressBook', 'Card', 'Preferences', 'stateComponent'];
+  function ComponentEditorController($rootScope, $scope, $q, $log, $timeout, $window, $element, $mdDialog, $mdToast, focus, User, CalendarSettings, Calendar, Component, Attendees, AddressBook, Card, Preferences, stateComponent) {
     var vm = this, component, oldStartDate, oldEndDate, oldDueDate, dayStartTime, dayEndTime;
 
     this.$onInit = function () {
@@ -280,7 +281,7 @@
     this.changeCalendar = function () {
       var updateRequired = (this.component.attendees && this.component.attendees.length > 0);
       if (updateRequired)
-        this.component.initOrganizer(Calendar.$get(this.component.destinationCalendar));
+        this.component.$attendees.initOrganizer(Calendar.$get(this.component.destinationCalendar));
     };
 
     // Autocomplete cards for attendees
@@ -291,7 +292,8 @@
     this.addAttendee = function (card, partial) {
       var initOrganizer = (!this.component.attendees || this.component.attendees.length === 0),
           destinationCalendar = Calendar.$get(this.component.destinationCalendar),
-          options = initOrganizer? { organizerCalendar: destinationCalendar } : {};
+          options = initOrganizer? { organizerCalendar: destinationCalendar } : {},
+          promises = [];
       var emailRE = /([\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+\.)*[\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+@((((([a-z0-9]{1}[a-z0-9\-]{0,62}[a-z0-9]{1})|[a-z])\.)+[a-z]{2,})|(\d{1,3}\.){3}\d{1,3}(\:\d{1,5})?)/i,
           i, address;
       if (partial) options.partial = partial;
@@ -316,7 +318,7 @@
 
       function addCard(newCard) {
         if (!vm.component.$attendees.hasAttendee(newCard))
-          vm.component.$attendees.add(newCard, options);
+          return vm.component.$attendees.add(newCard, options);
       }
 
       if (angular.isString(card)) {
@@ -332,23 +334,27 @@
                card.charCodeAt(i) == 44 ||   // ,
                card.charCodeAt(i) == 59) &&  // ;
               emailRE.test(address)) {
-            createCard(address).then(addCard);
+            promises.push(createCard(address).then(addCard));
             address = '';
           }
           else {
             address += card.charAt(i);
           }
         }
-        if (address && emailRE.test(address))
-          createCard(address).then(addCard);
+        if (address && emailRE.test(address)) {
+          promises.push(createCard(address).then(addCard));
+        }
       }
       else if (angular.isDefined(card)) {
         if (!this.component.$attendees.hasAttendee(card))
-          this.component.$attendees.add(card, options);
+          promises.push(this.component.$attendees.add(card, options));
         this.showAttendeesEditor |= initOrganizer;
       }
 
-      $timeout(scrollToStart);
+      if (_.has(this.component, '$attendees'))
+        $timeout(scrollToStart);
+
+      return $q.all(promises);
     };
 
     function scrollToStart() {
@@ -466,20 +472,21 @@
       this.adjustStartTime();
       this.adjustEndTime();
       this.changeAlarmRelation(form);
-      if (form.$valid) {
-        this.component.$save(options)
-          .then(function(data) {
-            $rootScope.$emit('calendars:list');
-            Preferences.getAlarms();
-            $mdDialog.hide();
-          }, function(response) {
-            if (response.status == CalendarSettings.ConflictHTTPErrorCode &&
-                _.isObject(response.data.message))
-              vm.attendeeConflictError = response.data.message;
-            else
-              vm.edit(form);
-          });
-      }
+      this.addAttendee(this.searchText).then(function () {
+        if (form.$valid) {
+          vm.component.$save(options)
+            .then(function(data) {
+              $rootScope.$emit('calendars:list');
+              Preferences.getAlarms();
+              $mdDialog.hide();
+            }, function(response) {
+              if (response.status == CalendarSettings.ConflictHTTPErrorCode)
+                vm.attendeeConflictError = _.isObject(response.data.message) ? response.data.message : { reject: response.data.message };
+              else
+                vm.edit(form);
+            });
+        }
+      });
     };
 
     this.reset = function (form) {
@@ -623,9 +630,11 @@
     };
 
     function updateFreeBusy() {
-      vm.component.$attendees.updateFreeBusyCoverage();
-      vm.component.$attendees.updateFreeBusy();
-      $timeout(scrollToStart);
+      if (_.has(vm.component, '$attendees')) {
+        vm.component.$attendees.updateFreeBusyCoverage();
+        vm.component.$attendees.updateFreeBusy();
+        $timeout(scrollToStart);
+      }
     }
   }
 
